/*
 Wireless Gameboy/GBA Controller - USB RX
 Version: 1.3
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 23/09/2018
 Last Modified: 16/03/2019
 
 */

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1

#define LED PA2

#define GB_MODE 1
#define GBA_MODE 2

// nRF settings (loaded from EEPROM)
uint8_t nrfChannel = 0;
uint8_t nrfRx[5] = "wgbr"; // Last byte loaded from EEPROM

// EEPROM settings
#define nrfChannelLocation 1
#define nrfRxLocation 2
#define blankEEPROMByte 255


extern void mirf_init(void);
extern void mirf_config(void);

// SPI transfer 1 byte and return the result
uint8_t spi_transfer(uint8_t data) {
	SPDR = data;
	
	// Wait for transmission complete
	while (!(SPSR & (1<<SPIF)));
	
	return SPDR;
}

// Write data using SPI
void spi_write_data(uint8_t * dataout, uint8_t len) {
	uint8_t i;
	for (i = 0; i < len; i++) {
		spi_transfer(dataout[i]);
	}
}

// Read data using SPI
void spi_read_data(uint8_t * datain, uint8_t len) {
	uint8_t i;
	for (i = 0; i < len; i++) {
		datain[i] = spi_transfer(0x00);
	}
}

// Initialise the SPI
void spi_init(void) {
	DDRA |= (1<<PA4); // SPI CLK
	DDRA &= ~(1<<PA5); // SPI DI
	PORTA |= (1<<PA5); // SPI DI
	DDRA |= (1<<PA6); // SPI DO
	DDRA |= (1<<PA7); // SPI SS
	
	// Enable SPI, Master, set clock rate fck/4
	SPCR = (1<<SPE)|(1<<MSTR);
}

// Setup
void setup(void) {
	DDRA |= (1<<LED);
	PORTA |= (1<<LED);
	_delay_ms(250); // Wait a little bit after power on
	PORTA &= ~(1<<LED);
	
	// Load nRF channel and address from EEPROM
	nrfChannel = eeprom_read_byte((uint8_t*) nrfChannelLocation);
	nrfRx[4] = eeprom_read_byte((uint8_t*) nrfRxLocation);
	if (nrfChannel == blankEEPROMByte) {
		nrfChannel = 1;
		nrfRx[4] = 1;
	}
	
	spi_init(); // Initialise SPI
	sei(); // Turn on interrupts
	
	mirf_init();
	_delay_ms(50);
	mirf_config();
}


// ******* Begin V-USB required code ********
const PROGMEM char usbHidReportDescriptor[42] = {
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x05,                    // USAGE (Game Pad)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xa1, 0x00,                    //   COLLECTION (Physical)
    0x09, 0x30,                    //     USAGE (X)
    0x09, 0x31,                    //     USAGE (Y)
    0x15, 0x81,                    //     LOGICAL_MINIMUM (-127)
    0x25, 0x7f,                    //     LOGICAL_MAXIMUM (127)
    0x75, 0x08,                    //     REPORT_SIZE (8)
    0x95, 0x02,                    //     REPORT_COUNT (2)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0xc0,                          //   END_COLLECTION
	 
    0x05, 0x09,                    // USAGE_PAGE (Button)
    0x19, 0x01,                    //   USAGE_MINIMUM (Button 1)
    0x29, 0x10,                    //   USAGE_MAXIMUM (Button 16)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x10,                    //   REPORT_COUNT (12)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0xc0                           // END_COLLECTION
};

usbMsgLen_t usbFunctionSetup(uchar data[8]) {
  usbRequest_t *req = (void *)data;

  return 0; // Nothing implemented
}

void usbEventResetReady(void) {
	//calibrateOscillator();
}

void hadUsbReset(void) {
  //calibrateOscillator();
  //eeprom_write_byte(0, OSCCAL);   /* store the calibrated value in EEPROM */
}

void startUSB(void) {
	usbDeviceDisconnect();
	_delay_ms(300); // 300 ms disconnect
	usbDeviceConnect();
	usbInit();
}